#include "dynamic-input-device-provider.h"

static void
detect_ring_device (std::vector<Ekiga::InputDevice::DescriptorPtr>& descriptors)
{
  Ekiga::InputDevice::DescriptorPtr descriptor;

  descriptor = Ekiga::InputDevice::DescriptorPtr(new Ekiga::InputDevice::Descriptor);
  descriptor->display_name = "Repeated wav file"; // FIXME: translate? It's supposed to go
  descriptor->provider = DYNAMIC_INPUT_DEVICE_PROVIDER_NAME;
  descriptor->description = "multifilesrc location=/usr/share/sounds/alsa/Noise.wav loop=true ! wavparse ignore-length=true ! audioconvert name=local_audio_hook";
  descriptor->device_type = Ekiga::InputDevice::Audio;
  descriptors.push_back (descriptor);
}

static void
detect_audio_devices (std::vector<Ekiga::InputDevice::DescriptorPtr>& descriptors)
{
  detect_ring_device (descriptors);
}

static void
detect_v4l2_device (std::vector<Ekiga::InputDevice::DescriptorPtr>& descriptors)
{
  GstElement* elt = gst_element_factory_make ("v4l2src", "v4l2srcpresencetest");
  if (elt != NULL) {

    Ekiga::InputDevice::DescriptorPtr descriptor;

    descriptor = Ekiga::InputDevice::DescriptorPtr(new Ekiga::InputDevice::Descriptor);
    descriptor->display_name = "V4L2 source"; // FIXME: translate? It's supposed to go
    descriptor->provider = DYNAMIC_INPUT_DEVICE_PROVIDER_NAME;
    descriptor->description = "v4l2src name=local_video_hook";
    descriptor->device_type = Ekiga::InputDevice::Video;
    descriptors.push_back (descriptor);
    gst_object_unref (GST_OBJECT (elt));
  }
}

static void
detect_ximage_device (std::vector<Ekiga::InputDevice::DescriptorPtr>& descriptors)
{
  GstElement* elt = gst_element_factory_make ("ximagesrc", "ximagesrcpresencetest");
  if (elt != NULL) {

    Ekiga::InputDevice::DescriptorPtr descriptor;

    descriptor = Ekiga::InputDevice::DescriptorPtr(new Ekiga::InputDevice::Descriptor);
    descriptor->display_name = "Export display"; // FIXME: translate? It's supposed to go
    descriptor->provider = DYNAMIC_INPUT_DEVICE_PROVIDER_NAME;
    descriptor->description = "ximagesrc name=local_video_hook";
    descriptor->device_type = Ekiga::InputDevice::Video;
    descriptors.push_back (descriptor);
    gst_object_unref (GST_OBJECT (elt));
  }
}

static void
detect_video_devices (std::vector<Ekiga::InputDevice::DescriptorPtr>& descriptors)
{
  detect_v4l2_device (descriptors);
  detect_ximage_device (descriptors);
}

Ekiga::DynamicInputDeviceProvider::DynamicInputDeviceProvider()
{
  detect_video_devices (descriptors);
  detect_audio_devices (descriptors);
}

Ekiga::DynamicInputDeviceProvider::~DynamicInputDeviceProvider()
{
}

void
Ekiga::DynamicInputDeviceProvider::add_input_device (Ekiga::MediaSessionPtr session,
						    InputDevice::DescriptorPtr descriptor)
{
  GstElement* element = gst_parse_bin_from_description (descriptor->description.c_str (),
							FALSE, NULL);
  gst_element_set_name (element, descriptor->display_name.c_str ());
  session->add_input_device (element);
  devices.push_front (boost::make_tuple(boost::weak_ptr<Ekiga::MediaSession>(session),
					descriptor, element));

  // should be reasonable
  if (devices.size () > 0 && devices.size () % 10 == 0)
    cleanup_dead_sessions_devices ();
}

void
Ekiga::DynamicInputDeviceProvider::remove_input_device (Ekiga::MediaSessionPtr session,
						       InputDevice::DescriptorPtr descriptor)
{
  devices_type::iterator iter = devices.begin ();

  for (; iter != devices.end (); ++iter) {

    Ekiga::MediaSessionPtr other_session = (*iter).get<0> ().lock ();
    InputDevice::DescriptorPtr other_descriptor = (*iter).get<1> ();
    /* IMPORTANT: the gui may have rebuilt the descriptor to match
     * this data, so we have to actually check for it instead of going
     * the easy way of other_descriptor == descriptor!
     */
    if (other_session == session
	&& other_descriptor->provider == descriptor->provider
	&& other_descriptor->description == descriptor->description
	&& other_descriptor->device_type == descriptor->device_type)
      break;
  }
  if (iter != devices.end ()) {

    session->remove_input_device ((*iter).get<2> ());
    devices.erase (iter);
  }
}

bool
Ekiga::DynamicInputDeviceProvider::visit_devices (boost::function1<bool, InputDevice::DescriptorPtr> visitor) const
{
  bool go_on = true;
  for (std::vector<InputDevice::DescriptorPtr>::const_iterator iter = descriptors.begin ();
       iter != descriptors.end () and go_on;
       ++iter) {

    go_on = visitor (*iter);
  }
  return go_on;
}

void
Ekiga::DynamicInputDeviceProvider::cleanup_dead_sessions_devices ()
{
  devices_type::iterator iter = devices.begin ();
  while (iter != devices.end ()) {

    Ekiga::MediaSessionPtr session = (*iter).get<0> ().lock ();

    if (!session) // He's dead, Jim!
      devices.erase (iter);
    else
      ++iter;
  }
}
