#include "media-input.h"

#include "static-input-device-provider.h"
#include "dynamic-input-device-provider.h"

Ekiga::MediaInput::MediaInput (Ekiga::MediaCorePtr core)
{
  connection = core->session_added.connect (boost::bind (&Ekiga::MediaInput::add_input_devices, this, _1));

  boost::shared_ptr<StaticInputDeviceProvider> provider (new StaticInputDeviceProvider);
  add_provider (boost::dynamic_pointer_cast<InputDevice::Provider> (provider));
  chosen_audio_device = provider->descriptors[0];
  chosen_video_device = provider->descriptors[1];
  add_provider (boost::shared_ptr<InputDevice::Provider> (new DynamicInputDeviceProvider));
}

Ekiga::MediaInput::~MediaInput ()
{
  connection.disconnect ();
}

void
Ekiga::MediaInput::add_provider (boost::shared_ptr<InputDevice::Provider> provider)
{
  provider->visit_devices (boost::bind(&Ekiga::MediaInput::visit_provider_devices, this, _1));
  providers.push_front (provider);
}

bool
Ekiga::MediaInput::visit_devices (boost::function1<bool, InputDevice::DescriptorPtr> visitor) const
{
  bool go_on = true;

  for (std::list<boost::shared_ptr<InputDevice::Provider> >::const_iterator iter = providers.begin ();
       iter != providers.end () and go_on;
       ++iter) {

    go_on = (*iter)->visit_devices (visitor);
  }
  return go_on;
}

void
Ekiga::MediaInput::set_default_device (InputDevice::DescriptorPtr descriptor)
{
  MediaSessionPtr session_ = session.lock();

  // there is a live call
  if (session_) {

    InputDevice::DescriptorPtr current;
    providers_type::iterator iter;
    boost::shared_ptr<InputDevice::Provider> current_provider;
    boost::shared_ptr<InputDevice::Provider> next_provider;

    switch (descriptor->device_type) {

    case InputDevice::Audio:
      current = chosen_audio_device;
      break;

    case InputDevice::Video:
      current = chosen_video_device;
      break;
    }
    for (iter = providers.begin (); iter != providers.end (); ++iter) {

      if ((*iter)->get_name () == current->provider)
	break;
    }
    current_provider = *iter;
    for (iter = providers.begin (); iter != providers.end (); ++iter) {

      if ((*iter)->get_name () == descriptor->provider)
	break;
    }
    next_provider = *iter;

    next_provider->add_input_device (session_, descriptor);
    current_provider->remove_input_device (session_, current);
  }

  switch (descriptor->device_type) {

  case InputDevice::Audio:
    chosen_audio_device = descriptor;
    break;

  case InputDevice::Video:
    chosen_video_device = descriptor;
    break;
  }
}

Ekiga::InputDevice::DescriptorPtr
Ekiga::MediaInput::get_default_device (InputDevice::Type device_type)
{
  InputDevice::DescriptorPtr result;

  switch (device_type) {

  case InputDevice::Audio:
    result = chosen_audio_device;
    break;

  case InputDevice::Video:
    result = chosen_video_device;
    break;
  }

  return result;
}

bool
Ekiga::MediaInput::visit_provider_devices (InputDevice::DescriptorPtr descriptor)
{
  on_device_added (descriptor);
  return true;
}

void
Ekiga::MediaInput::add_input_devices (Ekiga::MediaSessionPtr session_)
{
  session = session_;
  providers_type::iterator iter;

  for (iter = providers.begin (); iter != providers.end (); ++iter) {

    if ((*iter)->get_name () == chosen_audio_device->provider)
      break;
  }
  (*iter)->add_input_device (session_, chosen_audio_device);

  for (iter = providers.begin (); iter != providers.end (); ++iter) {

    if ((*iter)->get_name () == chosen_video_device->provider)
      break;
  }
  (*iter)->add_input_device (session_, chosen_video_device);
}

void
Ekiga::MediaInput::on_device_added (InputDevice::DescriptorPtr descriptor)
{
  device_added (descriptor);
}

void
Ekiga::MediaInput::on_device_removed (InputDevice::DescriptorPtr descriptor)
{
  // FIXME: here we should react if a device we're supposed to use is gone
  device_removed (descriptor);
}
