#include "media-core.h"

#ifndef __MEDIA_INPUT_H__
#define __MEDIA_INPUT_H__

namespace Ekiga {

  namespace InputDevice {
  enum Type
    {
      Audio,
      Video
    };

  struct Descriptor
  {
    std::string display_name; // directed to the user
    std::string provider; // responsible provider by its name
    std::string description; // provider-directed description of the device
    Type device_type;
  };
  typedef boost::shared_ptr<Descriptor> DescriptorPtr;

  /* Abstract class, which will make it possible to have several
   * sources of devices ; I(Snark) see the need for at least three:
   *
   * - one for the static devices (audio and video test)
   *
   * - one for the dynamically detected of devices (live devices like
   *   webcams or the user display) [FIXME: do it here]
   *
   * - one for stored devices (user-created ones like recorded audio
   * messages or logo) [TODO in ekiga]
   *
   */
  class Provider
  {
  public:

    virtual ~Provider()
    {}

    // The same name as used in the input device descriptions
    virtual const std::string get_name () const = 0;

    /* This method will be used by the Ekiga::MediaInput class to add
     * the user-chosen input devices to a new call, or to change the
     * input devices during a call
     */
    virtual void add_input_device (Ekiga::MediaSessionPtr session,
				   DescriptorPtr descriptor) = 0;

    /* This method will be used by the Ekiga::MediaInput class to
     * remove input devices if they change during a call.
     */
    virtual void remove_input_device (Ekiga::MediaSessionPtr session,
				      DescriptorPtr descriptor) = 0;

    /* This method makes it possible to find the available devices
     *
     * @return: the last value returned by the visitor
     */
    virtual bool visit_devices (boost::function1<bool, InputDevice::DescriptorPtr> visitor) const = 0;

    boost::signals2::signal<void(DescriptorPtr)> device_added;
    boost::signals2::signal<void(DescriptorPtr)> device_removed;
  };
  };

  /* The main working class, on which the gui will work ; its role
   * towards the user is to provide the list of available input
   * devices, to choose which ones to use when starting a call, or to
   * switch to during a call. It will also notify when input devices
   * appear or disappear.
   */
  class MediaInput
  {
  public:

    MediaInput (Ekiga::MediaCorePtr core);

    ~MediaInput ();

    void add_provider (boost::shared_ptr<InputDevice::Provider> provider);

    bool visit_devices (boost::function1<bool, InputDevice::DescriptorPtr> visitor) const;

    void set_default_device (InputDevice::DescriptorPtr descriptor);
    InputDevice::DescriptorPtr get_default_device (InputDevice::Type device_type);

    boost::signals2::signal<void(InputDevice::DescriptorPtr)> device_added;
    boost::signals2::signal<void(InputDevice::DescriptorPtr)> device_removed;

  private:
    // manage the providers
    typedef std::list<boost::shared_ptr<InputDevice::Provider> > providers_type;
    providers_type providers;
    InputDevice::DescriptorPtr chosen_audio_device;
    InputDevice::DescriptorPtr chosen_video_device;

    // FIXME: temporary
    bool visit_provider_devices (InputDevice::DescriptorPtr descriptor);
    void on_device_added (InputDevice::DescriptorPtr descriptor);
    void on_device_removed (InputDevice::DescriptorPtr descriptor);

    // manage the session
    boost::weak_ptr<MediaSession> session;
    boost::signals2::connection connection;
    void add_input_devices (MediaSessionPtr session_);
  };
}

#endif
