#include "media-session.h"

#define DEBUG 1

#if DEBUG
static void
dump (GstElement* pipeline)
{
  static int num = 0;
  gchar* name = g_strdup_printf("pipeline-%d", num);

  GST_DEBUG_BIN_TO_DOT_FILE (GST_BIN (pipeline), GST_DEBUG_GRAPH_SHOW_ALL, name);
  g_free (name);
  num++;
}
#endif

static bool
link_elements (GstElement* first,
	       const std::string first_hook_name,
	       GstElement* second,
	       const std::string second_hook_name)
{
  bool result = true;
  GstElement* first_hook = gst_bin_get_by_name (GST_BIN (first), first_hook_name.c_str ());
  GstElement* second_hook = gst_bin_get_by_name (GST_BIN (second), second_hook_name.c_str ());

  if (first_hook && second_hook)
    result = gst_element_link (first_hook, second_hook);

  if (first_hook)
    gst_object_unref (first_hook);
  if (second_hook)
    gst_object_unref (second_hook);

  return result;
}

static bool
unlink_from_mixer (GstElement* mixer,
		   GstElement* device)
{
  bool result = false;
  GstIterator* iter = gst_element_iterate_sink_pads (mixer);
  GstPad* sink_pad = NULL;
  GstPad* src_pad = NULL;
  GstElement* src_parent = NULL;
  GValue item = G_VALUE_INIT;
  bool done = false;

  while (!done) {

    switch (gst_iterator_next (iter, &item)) {

    case GST_ITERATOR_OK: {
      sink_pad = (GstPad*)g_value_get_object (&item);
      src_pad = gst_pad_get_peer (sink_pad);

      if (GST_IS_PROXY_PAD (src_pad)) {

	GstPad* other = (GstPad*)gst_proxy_pad_get_internal (GST_PROXY_PAD (src_pad));
	GstPad* real = gst_pad_get_peer (other);
	gst_object_unref (src_pad);
	src_pad = real;
	gst_object_unref (other);
      }

      src_parent = (GstElement*)gst_pad_get_parent (src_pad);

      if (src_parent == device) {

	gst_pad_send_event (sink_pad, gst_event_new_eos ());
	gst_element_remove_pad (mixer, sink_pad);
	done = true;
	result = true;
      }
      gst_object_unref (src_parent);
      gst_object_unref (src_pad);
      break;
    }

    case GST_ITERATOR_DONE:
      done = true;
      break;

    case GST_ITERATOR_RESYNC:
      gst_iterator_resync (iter);
      break;

    case GST_ITERATOR_ERROR:
      done = true;
      break;
    }
    g_value_reset (&item);
  }

  return result;
}

Ekiga::MediaSession::MediaSession ()
{
  // FIXME: what should we do to handle errors cleanly?
  GError* error = NULL;
  GstElement* tee = NULL;
  GstElement* queue = NULL;

  pipeline = gst_pipeline_new ("Main pipeline");

  GstElement* media_hub =
    gst_parse_bin_from_description (" adder name=local_audio_mixer "
				    "! tee name=local_audio_tee "
				    " videomixer name=local_video_mixer "
				    "! tee name=local_video_tee "
				    " adder name=remote_audio_mixer "
				    "! tee name=remote_audio_tee "
				    " videomixer name=remote_video_mixer "
				    "! tee name=remote_video_tee "
				    " ", FALSE, &error);
  if (error) {

    GST_WARNING ("%s failed: %s\n", __PRETTY_FUNCTION__, error->message);
    g_error_free (error);
  }
  gst_element_set_name (media_hub, "Media hub");
  gst_bin_add (GST_BIN (pipeline), media_hub);

  GstElement* recorder_hook
    = gst_parse_bin_from_description (" queue name=record_local_audio ! audioconvert "
				      "! interaudiosink channel=local_audio "
				      "  queue name=record_local_video ! videoconvert "
				      "! intervideosink channel=local_video "
				      " queue name=record_remote_audio ! audioconvert "
				      "! interaudiosink channel=remote_audio "
				      " queue name=record_remote_video ! videoconvert "
				      "! intervideosink channel=remote_video "
				      " ", FALSE, &error);
  if (error) {

    GST_WARNING ("%s failed: %s\n", __PRETTY_FUNCTION__, error->message);
    g_error_free (error);
  }
  gst_element_set_name (recorder_hook, "Recorder hook");
  gst_bin_add (GST_BIN (pipeline), recorder_hook);

  tee = gst_bin_get_by_name (GST_BIN (media_hub), "local_audio_tee");
  queue = gst_bin_get_by_name (GST_BIN (recorder_hook), "record_local_audio");
  gst_element_link (tee, queue);
  gst_object_unref (tee);
  gst_object_unref (queue);

  tee = gst_bin_get_by_name (GST_BIN (media_hub), "local_video_tee");
  queue = gst_bin_get_by_name (GST_BIN (recorder_hook), "record_local_video");
  gst_element_link (tee, queue);
  gst_object_unref (tee);
  gst_object_unref (queue);

  tee = gst_bin_get_by_name (GST_BIN (media_hub), "remote_audio_tee");
  queue = gst_bin_get_by_name (GST_BIN (recorder_hook), "record_remote_audio");
  gst_element_link (tee, queue);
  gst_object_unref (tee);
  gst_object_unref (queue);

  tee = gst_bin_get_by_name (GST_BIN (media_hub), "remote_video_tee");
  queue = gst_bin_get_by_name (GST_BIN (recorder_hook), "record_remote_video");
  gst_element_link (tee, queue);
  gst_object_unref (tee);
  gst_object_unref (queue);

  recorder
    = gst_parse_launch (" intervideosrc channel=local_video "
			"! videoconvert "
			"! ximagesink "
			" intervideosrc channel=remote_video "
			"! videoconvert "
			"! ximagesink "
			" ", &error);
  if (error) {

    GST_WARNING ("%s failed: %s\n", __PRETTY_FUNCTION__, error->message);
    g_error_free (error);
  }
  gst_element_set_name (recorder, "Recorder pipeline");
}

Ekiga::MediaSession::~MediaSession ()
{
  gst_element_set_state (pipeline, GST_STATE_NULL);
  gst_object_unref (pipeline);

  gst_element_set_state (recorder, GST_STATE_NULL);
  gst_object_unref (recorder);
}

bool
Ekiga::MediaSession::add_input_device (GstElement* device)
{
  bool result = true;
  if (result and (gst_bin_add (GST_BIN (pipeline), device) == TRUE)) {

    result =
      true
      and link_elements (device, "local_audio_hook", pipeline, "local_audio_mixer")
      and link_elements (device, "local_video_hook", pipeline, "local_video_mixer")
      and link_elements (device, "remote_audio_hook", pipeline, "remote_audio_mixer")
      and link_elements (device, "remote_video_hook", pipeline, "remote_video_mixer");

    if ( !result)
      gst_bin_remove (GST_BIN (pipeline), device);
    else
      gst_element_sync_state_with_parent (device);
  }

  return result;
}

bool
Ekiga::MediaSession::add_output_device (GstElement* device)
{
  bool result = false;

  if (is_running)
    gst_element_set_state (device, GST_STATE_PLAYING);

  if (gst_bin_add (GST_BIN (pipeline), device)) {

    result =
      true
      and link_elements (pipeline, "local_audio_tee", device, "local_audio_hook")
      and link_elements (pipeline, "local_video_tee", device, "local_video_hook")
      and link_elements (pipeline, "remote_audio_tee", device, "remote_audio_hook")
      and link_elements (pipeline, "remote_video_tee", device, "remote_video_hook");

    if ( !result)
      gst_bin_remove (GST_BIN (pipeline), device);
  }
  return result;
}

bool
Ekiga::MediaSession::remove_input_device (GstElement* device)
{
  GstElement* mixer = NULL;

  { // local audio
    mixer = gst_bin_get_by_name (GST_BIN (pipeline), "local_audio_mixer");
    unlink_from_mixer (mixer, device);
    gst_object_unref (mixer);
  }
  { // local video
    mixer = gst_bin_get_by_name (GST_BIN (pipeline), "local_video_mixer");
    unlink_from_mixer (mixer, device);
    gst_object_unref (mixer);
  }
  { // remote audio
    mixer = gst_bin_get_by_name (GST_BIN (pipeline), "remote_audio_mixer");
    unlink_from_mixer (mixer, device);
    gst_object_unref (mixer);
  }
  { // remote video
    mixer = gst_bin_get_by_name (GST_BIN (pipeline), "remote_video_mixer");
    unlink_from_mixer (mixer, device);
    gst_object_unref (mixer);
  }
  gst_element_set_state (device, GST_STATE_NULL);
  gst_bin_remove (GST_BIN (pipeline), device);

  return true;
}

void
Ekiga::MediaSession::start_media_pipeline ()
{
  gst_element_set_state (pipeline, GST_STATE_PLAYING);
  is_running = true;
#if DEBUG
  dump (pipeline);
#endif
}

void
Ekiga::MediaSession::stop_media_pipeline ()
{
  gst_element_set_state (pipeline, GST_STATE_NULL);
  is_running = false;
}

void
Ekiga::MediaSession::start_recording ()
{
  if (is_running) {

    gst_element_set_state (recorder, GST_STATE_PLAYING);
    is_recording = true;
  }
}

void
Ekiga::MediaSession::stop_recording ()
{
  if (is_running) {

    gst_element_set_state (recorder, GST_STATE_NULL);
    is_recording = false;
  }
}

static void
on_eos (G_GNUC_UNUSED GstBus* bus,
	G_GNUC_UNUSED GstMessage* msg,
	GstElement* pipeline)
{
  gst_element_set_state (pipeline, GST_STATE_NULL);
  gst_object_unref (pipeline);
}

void
Ekiga::MediaSession::play_file_once (const std::string filename)
{
  gchar* descr = g_strdup_printf ("filesrc location=\"%s\" ! wavparse ! goom ! ximagesink",
				  filename.c_str ());
  GError* error = NULL;
  GstElement* player = gst_parse_launch (descr, &error);
  g_free (descr);
  if (error) {

    GST_WARNING ("%s failed: %s\n", __PRETTY_FUNCTION__, error->message);
    g_error_free (error);
  } else {

    GstBus* bus = gst_element_get_bus (pipeline);
    gst_bus_add_signal_watch (bus);
    g_signal_connect (bus, "message::eos", G_CALLBACK (on_eos), player);
    gst_object_unref (bus);

    gst_element_set_state (player, GST_STATE_PLAYING);
  }
}

void
Ekiga::MediaSession::play_file_repeat (const std::string filename)
{
  stop_file_repeat (filename);

  gchar* descr = g_strdup_printf ("multifilesrc location=\"%s\" loop=1 ! wavparse ! goom ! ximagesink",
				  filename.c_str ());
  GError* error = NULL;
  GstElement* player = gst_parse_launch (descr, &error);
  g_free (descr);
  if (error) {

    GST_WARNING ("%s failed: %s\n", __PRETTY_FUNCTION__, error->message);
    g_error_free (error);
  } else {

    gst_element_set_state (player, GST_STATE_PLAYING);
    repeating_play_files[filename] = boost::shared_ptr<GstElement>(player,
								   gst_element_deleter ());
  }
}

void
Ekiga::MediaSession::stop_file_repeat (const std::string filename)
{
  std::map<std::string, boost::shared_ptr<GstElement> >::iterator iter
    = repeating_play_files.find (filename);

  if (iter != repeating_play_files.end ())
    repeating_play_files.erase (iter);
}
