#include "media-core.h"
#include "media-input.h"
#include "media-opal.h"
#include "gtk-combo-input-device.h"

#include <gtk/gtk.h>

struct application_state {

  Ekiga::MediaCorePtr core;
  boost::shared_ptr<Ekiga::MediaInput> input;
  Ekiga::MediaSession* session;
  GtkListStore* store;
  GtkWidget* start_recording_button;
  GtkWidget* stop_recording_button;
};

static void
add_dummy_output_devices (application_state* state,
			  Ekiga::MediaSessionPtr session_)
{
  state->session = session_.get ();
  {
    GstElement* device = gst_parse_bin_from_description ("queue name=local_video_hook ! ximagesink", FALSE, NULL);
    gst_element_set_name (device, "Local video output");
    state->session->add_output_device (device);
  }
  {
    GstElement* device = gst_parse_bin_from_description ("queue name=remote_video_hook ! ximagesink", FALSE, NULL);
    gst_element_set_name (device, "Remote video output");
    state->session->add_output_device (device);
  }
  {
    GstElement* device = gst_parse_bin_from_description ("queue name=local_audio_hook ! autoaudiosink", FALSE, NULL);
    gst_element_set_name (device, "Local audio output");
    state->session->add_output_device (device);
  }
}

/* helper code for the buttons */

static GtkWidget*
add_button (GtkWidget* box,
	    const gchar* label,
	    GCallback callback,
	    application_state* state)
{
  GtkWidget* button = gtk_button_new_with_label (label);
  g_signal_connect (button, "clicked", callback, (gpointer)state);
  gtk_box_pack_start (GTK_BOX (box), button,
		      TRUE, TRUE, 2);
  return button;
}

static void
start_call (GtkButton* button,
	    application_state* state)
{
  g_print ("Start call\n");
  state->core->new_call_trigger ();
  gtk_widget_set_sensitive (GTK_WIDGET (button), FALSE);
  gtk_widget_set_sensitive (state->start_recording_button, TRUE);
}

static void
start_recording (G_GNUC_UNUSED GtkButton* button,
		 application_state* state)
{
  g_print ("Start recording\n");
  state->session->start_recording ();
  gtk_widget_set_sensitive (state->start_recording_button, FALSE);
  gtk_widget_set_sensitive (state->stop_recording_button, TRUE);
}

static void
stop_recording (G_GNUC_UNUSED GtkButton* button,
		application_state* state)
{
  g_print ("Stop recording\n");
  state->session->stop_recording ();
  gtk_widget_set_sensitive (state->start_recording_button, TRUE);
  gtk_widget_set_sensitive (state->stop_recording_button, FALSE);
}

static void
quit (G_GNUC_UNUSED GtkButton* button,
      G_GNUC_UNUSED application_state* state)
{
  g_print ("Quitting\n");
  gtk_main_quit ();
}

static gboolean
on_delete_event (G_GNUC_UNUSED GtkWidget* widget,
		 G_GNUC_UNUSED GdkEvent* event,
		 G_GNUC_UNUSED gpointer data)
{
  return FALSE;
}

static void
on_destroy (G_GNUC_UNUSED GtkWidget* widget,
	    G_GNUC_UNUSED gpointer data)
{
  quit (NULL, NULL);
}

int
main (int argc,
      char* argv[])
{
  gst_init (&argc, &argv);
  gtk_init (&argc, &argv);

  application_state* state = new application_state;


  state->core = Ekiga::MediaCorePtr(new Ekiga::MediaCore);
  state->input = boost::shared_ptr<Ekiga::MediaInput>(new Ekiga::MediaInput (state->core));
  boost::shared_ptr<Opal::Media> opal(new Opal::Media (state->core));
  state->core->session_added.connect (boost::bind (&add_dummy_output_devices, state, _1));

  GtkWidget* window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title (GTK_WINDOW (window),
			"Media tests for ekiga");
  g_signal_connect (window, "delete-event", G_CALLBACK (on_delete_event), NULL);
  g_signal_connect (window, "destroy", G_CALLBACK (on_destroy), NULL);

  GtkWidget* box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 2);
  gtk_container_add (GTK_CONTAINER (window), box);

  GtkWidget* bbox = gtk_button_box_new (GTK_ORIENTATION_HORIZONTAL);
  gtk_container_add (GTK_CONTAINER (box), bbox);

  add_button (bbox, "start call", (GCallback)start_call, state);
  state->start_recording_button = add_button (bbox, "start recording", (GCallback)start_recording, state);
  gtk_widget_set_sensitive (state->start_recording_button, FALSE);
  state->stop_recording_button = add_button (bbox, "stop recording", (GCallback)stop_recording, state);
  gtk_widget_set_sensitive (state->stop_recording_button, FALSE);
  add_button (bbox, "quit", (GCallback)quit, state);

  GtkWidget* combo;
  combo = gtk_combo_input_device_new (state->input, Ekiga::InputDevice::Video);
  gtk_container_add (GTK_CONTAINER (box), combo);

  combo = gtk_combo_input_device_new (state->input, Ekiga::InputDevice::Audio);
  gtk_container_add (GTK_CONTAINER (box), combo);

  gtk_widget_show_all (window);

  gtk_main ();

  delete state;

  return 0;
}
